@@ -128,25 +128,39 @@ def wrapper%(wrapper_args)s:
 
 
 def get_wrapped(func, responses):
-    # Preserve the argspec for the wrapped function so that testing
-    # tools such as pytest can continue to use their fixture injection.
-    is_bound_method = hasattr(func, "__self__")
-
     if six.PY2:
         args, a, kw, defaults = inspect.getargspec(func)
         wrapper_args = inspect.formatargspec(args, a, kw, defaults)
-        if is_bound_method:
+
+        # Preserve the argspec for the wrapped function so that testing
+        # tools such as pytest can continue to use their fixture injection.
+        if hasattr(func, "__self__"):
             args = args[1:]  # Omit 'self'
         func_args = inspect.formatargspec(args, a, kw, None)
     else:
         signature = inspect.signature(func)
-        wrapper_args = str(signature)
-        if is_bound_method:
-            new_params = list(signature.parameters.values())[1:]  # omit 'self'
-            signature = signature.replace(parameters=new_params)
+        signature = signature.replace(return_annotation=inspect.Signature.empty)
+        # If the function is wrapped, switch to *args, **kwargs for the parameters
+        # as we can't rely on the signature to give us the arguments the function will
+        # be called with. For example unittest.mock.patch uses required args that are
+        # not actually passed to the function when invoked.
+        if hasattr(func, "__wrapped__"):
+            wrapper_params = [
+                inspect.Parameter("args", inspect.Parameter.VAR_POSITIONAL),
+                inspect.Parameter("kwargs", inspect.Parameter.VAR_KEYWORD),
+            ]
+        else:
+            wrapper_params = [
+                param.replace(annotation=inspect.Parameter.empty)
+                for param in signature.parameters.values()
+            ]
+        signature = signature.replace(parameters=wrapper_params)
 
+        wrapper_args = str(signature)
         params_without_defaults = [
-            param.replace(default=inspect.Parameter.empty)
+            param.replace(
+                annotation=inspect.Parameter.empty, default=inspect.Parameter.empty
+            )
             for param in signature.parameters.values()
         ]
         signature = signature.replace(parameters=params_without_defaults)
@@ -158,10 +172,7 @@ def get_wrapped(func, responses):
         evaldict,
     )
     wrapper = evaldict["wrapper"]
-
     update_wrapper(wrapper, func)
-    if is_bound_method:
-        wrapper = wrapper.__get__(func.__self__, type(func.__self__))
     return wrapper
 
 
@@ -379,6 +390,9 @@ class CallbackResponse(BaseResponse):
             raise result
 
         status, r_headers, body = result
+        if isinstance(body, Exception):
+            raise body
+
         body = _handle_body(body)
         headers.update(r_headers)
 
